use binrw::{BinRead, BinWrite, FilePtr64};

use crate::types::{Color, Double, Field, WString, Wobble, U32};

#[derive(BinRead, Debug)]
pub struct PenHeader {
    pub pen_count: u32,
    #[br(args {
        inner: binrw::args! {
            pen_count
        }
    })]
    pub start_offset: FilePtr64<Pens>,
}

#[derive(BinRead, BinWrite, Debug)]
#[br(import {pen_count: u32})]
pub struct Pens {
    // #[br(count = pen_count)]
    // pens: Vec<ArrayOf<Field>>,
    #[br(count = pen_count)]
    pub pens: Vec<Pen>,
}

#[derive(BinRead, BinWrite, Debug)]
pub struct Pen {
    #[br(assert(field_count == 236))]
    field_count: u32,
    pub color: Color,
    pub name: WString,
    pub disabled: U32,
    pub use_default: U32,
    pub loop_count: U32,
    pub speed: Double, // Changes with wobble relative speed
    pub power: Double,
    pub frequency: U32,
    _unknown_1: Field,
    pub start_tc: U32,
    pub end_tc: U32,
    pub polygon_tc: U32,
    pub jump_speed: Double,
    _unknown_2: [Field; 10],
    pub laser_off_tc: U32,
    pub wave: U32, // Only available if continue_mode is false
    _unknown_3: Field,
    pub wobble_enable: U32,
    pub wobble_diameter: Double,
    pub wobble_distance: Double,
    _unknown_4: [Field; 8],
    pub min_jump_tc: U32,
    pub max_jump_tc: U32,
    pub jump_limit: Double,
    _unknown_5: [Field; 2],
    pub frequency_2: Double,
    _unknown_6: [Field; 152],
    pub wobble_type: Wobble,
    pub continue_mode: U32,
    _unknown_7: [Field; 12],
    pub wobble_diameter_2: Double, // Only with wobble type ellipse
    _unknown_8: [Field; 26],
}
