#![allow(dead_code)]

use binrw::{binrw, BinRead, BinWrite};

use crate::{array_of::ArrayOf, field_of::FieldOf};

/// Generic field with structure of length + data
pub type Field = ArrayOf<u8>;

pub type U32 = FieldOf<u32>;
pub type U16 = FieldOf<u16>;
pub type Double = FieldOf<f64>;
pub type Color = FieldOf<Rgba>;
pub type Wobble = FieldOf<WobbleType>;

pub type WString = ArrayOf<u16>;

impl From<&WString> for String {
    fn from(value: &WString) -> Self {
        String::from_utf16_lossy(value)
    }
}

impl From<String> for WString {
    fn from(value: String) -> Self {
        let value: Vec<u16> = value.encode_utf16().collect();
        Self { value }
    }
}

#[derive(BinRead, BinWrite, PartialEq, Debug)]
pub struct Rgba {
    pub red: u8,
    pub green: u8,
    pub blue: u8,
    pub alpha: u8,
}

#[derive(BinRead, BinWrite, PartialEq, Debug)]
pub struct Point {
    pub x: f64,
    pub y: f64,
}

#[binrw]
#[derive(PartialEq, Debug)]
pub struct MultiLine {
    #[br(temp)]
    #[bw(try_calc(u32::try_from(points.len())))]
    point_count: u32,
    #[br(count = point_count)]
    points: Vec<Point>,
}

#[derive(BinRead, BinWrite, PartialEq, Debug)]
#[brw(repr(u32))]
#[repr(u32)]
pub enum WobbleType {
    Spiral = 0,
    Sinusoidal = 1,
    Ellipse = 2,
    Vert8 = 3,
    Hori8 = 4,
}

#[derive(BinRead, BinWrite, PartialEq, Debug)]
#[brw(repr(u32))]
#[repr(u32)]
pub enum ObjectType {
    Curve = 1,
    Point = 2,
    Rectangle = 3,
    Circle = 4,
    Ellipse = 5,
    Polygon = 6,
    Hatch = 32,
}

#[derive(BinRead, BinWrite, PartialEq, Debug)]
#[brw(repr(u16))]
#[repr(u16)]
pub enum LineType {
    Point = 0x0001,
    Line = 0x0100,
    Bezier = 0x0300,
}
